#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
Shared functions used along ``BacterialTyper`` & ``XICRA`` pipeline.
With different purposes:
    - Print time stamps
  
    - Create system calls

    - Manage/list/arrange files/folders

    - Aesthetics

    - Manage fasta files

    - Other miscellaneous functions
"""
## useful imports
from itertools import groupby
from termcolor import colored
import os

from HCGB.functions import system_call_functions

#########################
###  BLAST callers    ###
#########################

###############
def makeblastdb(DBname, fasta, makeblastDBexe, dbtype='nucl'):
    ## generate blastdb for genome
    if (os.path.isfile(DBname + '.nhr')):
        print ("+ BLAST database is already generated...")
    else:
        cmd_makeblast = "%s -in %s -input_type fasta -dbtype %s -out %s" %(makeblastDBexe, fasta, dbtype, DBname)
        code = system_call_functions.system_call(cmd_makeblast)

        if (code == 'FAIL'):
            print (colored('****ERROR: Some error happened during the makeblastDB command', 'red'))
            print (cmd_makeblast)
            exit()

###############    
def blastn(blastnexe, outFile, DBname, fasta, threads):
    # blastn 
    cmd_blastn = "%s -db %s -query %s -out %s -evalue 1e-20 -outfmt \'6 std qlen slen\' -num_threads %s" %(blastnexe, DBname, fasta, outFile, threads )
    codeBlastn = system_call_functions.system_call(cmd_blastn)
    
    if (codeBlastn == 'FAIL'):
        print (colored('****ERROR: Some error happened during the blastn command', 'red'))
        print (cmd_blastn)
        exit()

###############    
def blastp(blastpexe, outFile, DBname, fasta, threads):
    # blastp
    cmd_blastp = "%s -db %s -query %s -out %s -evalue 1e-20 -outfmt \'6 std qlen slen\' -num_threads %s" %(blastpexe, DBname, fasta, outFile, threads )
    codeBlastp = system_call_functions.system_call(cmd_blastp)
    
    if (codeBlastp == 'FAIL'):
        print (colored('****ERROR: Some error happened during the blastp command', 'red'))
        print (cmd_blastp)
        exit()

###############################
###  Parse BLAST results    ###
###############################

###########################################################################
#The original version was retrieved from:  
# - https://gist.github.com/aziele/e060d1c0d1ef2acb4bd67e2fc0f165a6
# - https://www.biostars.org/p/253984/
###########################################################################

class Hsp:
    """Store information about single HSP in an alignment hit. 
    
    :param qid: Query Id
    :param sid: Subject Id
    :param pident: Percentage of identical matches
    :param length: Alignment length
    :param mismatch: Number of mismatches
    :param gaps: Total number of gaps
    :param qstart: Start of alignment in query
    :param qend: End of alignment in query
    :param sstart: Start of alignment in subject
    :param send: End of alignment in subject
    :param evalue: Expect value
    :param bitscore: Bit score
    :param qlen: Query length
    :param slen: Subject length
    """

    def __init__(self,entry):
        bt_fields = entry.split('\t')
        self.qid = bt_fields[0]
        self.sid = bt_fields[1]
        self.pident = float(bt_fields[2])
        self.length = int(bt_fields[3])
        self.mismatch = int(bt_fields[4])
        self.gaps = int(bt_fields[5])
        self.qstart = int(bt_fields[6])
        self.qend = int(bt_fields[7])
        self.sstart = int(bt_fields[8])
        self.send = int(bt_fields[9])
        self.evalue = float(bt_fields[10])
        self.bitscore = float(bt_fields[11])
        self.qlen = float(bt_fields[12])
        self.slen = float(bt_fields[13])

    def _format(self):
        l = [self.qid, 
             self.sid, 
             '{0:.2f}'.format(self.pident),
             '{0}'.format(self.length),
             '{0}'.format(self.mismatch),
             '{0}'.format(self.gaps),
             '{0}'.format(self.qstart),
             '{0}'.format(self.qend),
             '{0}'.format(self.sstart),
             '{0}'.format(self.send),
             '{:.1E}'.format(self.evalue),
             '{0}'.format(self.bitscore),
             '{0}'.format(self.qlen),
             '{0}'.format(self.slen)
            ]
        return l

    def format(self):
        return '\t'.join(self._format())

    def __str__(self):
        f = "{0}\t{1}".format(self.qid, self.sid)
        return f

class BlastRecord:
    """Object representing a Blast Record. 
    
    :param qid: Query sequence id
    :param hits: Blast hits
    """

    def __init__(self, qid=None, hits=None):
        """Initialize Blast Record instance"""
        self.qid = qid
        self.hits = hits

    def evalue_cutoff(self, evalue):
        """Filter HSPs by given e-value."""
        l = []
        for hit in self.hits:
            hsps = []
            for hsp in hit:
                if hsp.evalue <= evalue:
                    hsps.append(hsp)
            if hsps: l.append(hsps)
        self.hits = l

    def best_hits(self):
        """Return list of first hits that obtain the 
           same score.
        """
        l = []
        if not self.hits: return l
        else:
            max_score = self.hits[0][0].bitscore
            for hit in self.hits:
                if hit[0].bitscore >= max_score:
                    l.append(hit)
                else:
                    break
            return l

    def best_hsps(self):
        """Return list of first HSPs that obtain the 
           same score.
        """
        l = []
        if not self.hits: return l
        else:
            max_score = self.hits[0][0].bitscore
            for hit in self.hits:
                if hit[0].bitscore >= max_score:
                    l.append(hit[0])
                else:
                    break
            return l

    def best_hsps_except_query(self):
        """Return list of first HSPs that obtain the 
           same score.
        """
        l = []
        if not self.hits: return l
        else:
            max_score = None
            for hit in self.hits:
                if hit[0].sid != self.qid:
                    if not max_score or hit[0].bitscore>=max_score:
                        max_score = hit[0].bitscore
                        l.append(hit[0])
                    else:
                        break
            return l

    def _format(self):
        l = []
        for hit in self.hits:
            for hsp in hit:
                l.append(hsp._format())
        return l        

    def format(self):
        """Return output string of BLAST record"""
        l = []
        for hit in self.hits:
            for hsp in hit:
                l.append(hsp.format())
            l.append('')
        return "\n".join(l).strip()

    def __str__(self):
        """Return output string of BLAST record"""
        return self.format()


#This is a generator function!
def parse(handle, eval_thresh=10, aln_thresh=0, length_thresh=0):
     """
     Script that parses BLAST output lists (in -outfmt '6 std qlen slen' format).

     The original version was retrieved from:  

     - https://gist.github.com/aziele/e060d1c0d1ef2acb4bd67e2fc0f165a6

     - https://www.biostars.org/p/253984/

     New implementation required query length (qlen) and subject lenght (slen) in the BLAST format input file. 
     It allows to parse for alignment & length threshold. 

     Generator function to iterate over Blast records.
     
     :param handle: input file handle containg Blast tabular output -outfmt '6 std qlen slen'
     :param eval_thresh: E-value cutoff for Blast results. (1-eX)
     :param aln_thresh: Alingment cutoff (%)
     :param length_thresh: Cutoff for a given length provided (bp)
     
     :type handle: string
     :type eval_thresh: integer
     :type aln_thresh: integer
     :type length_thresh: integer
     
     :returns: BlastRecord objetct.
     
     """
     for qid, blasts in groupby(handle, lambda l: l.split()[0]):
         hits = []
         prev_sid = False
         for sid, hsps in groupby(blasts, lambda l: l.split()[1]):
             hsps_temp = []
             for line in hsps:
#                 line = line.functions.decode("utf-8")
                 hsp = Hsp(line)
                 
                 if (hsp.qid == hsp.sid):
                     continue ## discard reporting autohit
                 
                 aln = (int(hsp.length)/int(hsp.qlen))*100
                 if aln >= aln_thresh and hsp.evalue <= eval_thresh and hsp.qlen > length_thresh:
                     hsps_temp.append(hsp)
             if hsps_temp: hits.append(hsps_temp)
         yield BlastRecord(qid=qid, hits=hits)



#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
Shared functions used along ``BacterialTyper`` & ``XICRA`` pipeline.
With different purposes:
    - Print time stamps
  
    - Create system calls

    - Manage/list/arrange files/folders

    - Aesthetics

    - Manage fasta files

    - Other miscellaneous functions
"""
## useful imports
from termcolor import colored
import patoolib ## to extract
import os

##
from HCGB.functions import system_call_functions

############################################################################
########                     FILES/FOLDERS                            ########                     
############################################################################

###############
def is_non_zero_file(fpath):  
    # https://stackoverflow.com/a/15924160
    """Returns TRUE/FALSE if file exists and non zero"""
    return os.path.isfile(fpath) and os.path.getsize(fpath) > 0

###############
def outdir_project(outdir, project_mode, pd_samples, mode, debug):
    """
    """
    # Group dataframe by sample name
    sample_frame = pd_samples.groupby(["new_name"])
    if (debug):
        print ("+ Dataframe grouby variable 'new_name'")

    dict_outdir = {}    
    for name, cluster in sample_frame:
        if (debug):
            print (cluster)
        if (project_mode):
            #print ("Create subdir for every sample: ", mode)
            sample_dir = create_subfolder('data', outdir)        

            ## create sample
            sample_name_dir = create_subfolder(name, sample_dir)        

            ## create subdir sub sample
            mode_name_dir = create_subfolder(mode, sample_name_dir)        
            dict_outdir[name] = mode_name_dir

        else:
            #print ("All samples share same folder")
            sample_name_dir = create_subfolder(name, outdir)        
            dict_outdir[name] = sample_name_dir

    return (dict_outdir)

###############
#def outdir_subproject(outdir, pd_samples, mode):
#    ## we assume we want to create within a project dir a subdir
#    # Group dataframe by sample name
#    sample_frame = pd_samples.groupby(["name"])
#    dict_outdir = {}    
#    for name, cluster in sample_frame:
#        mode_name_dir = create_subfolder(mode, outdir)        
#        dict_outdir[name] = mode_name_dir
#
#    return (dict_outdir)

###############
def create_subfolder (name, path):
    """Create a subfolder named 'name' in directory 'path'. Returns path created."""
    ## create subfolder  ##    
    subfolder_path = path + "/" + name
    access_rights = 0o755

    # define the access rights
    try:
        os.mkdir(subfolder_path, access_rights)
    except OSError:  
        #print ("\tDirectory %s already exists" % subfolder_path)
        return subfolder_path
    else:  
        print (colored("Successfully created the directory %s " % subfolder_path, 'yellow'))

    return subfolder_path

###############  
def create_folder (path):
    """Create a folder directory 'path'. Returns path created."""

    ## create subfolder  ##    
    access_rights = 0o755

    # define the access rights
    try:
        os.mkdir(path, access_rights)
    except OSError:  
        #print ("\tDirectory %s already exists" %path)
        return path
    else:  
        print (colored("Successfully created the directory %s " %path, 'yellow'))

    return path

############### 
def get_symbolic_link (sample_list, directory):
    """Creates symbolic links, using system call, for list of files given in directory provided"""
    for samplex in sample_list:
        cmd = 'ln -s %s %s' %(samplex, directory)
        system_call_functions.system_call(cmd, returned=False)

    files2return = os.listdir(directory)
    return files2return

###############
def get_symbolic_link_file (file2link, newfile):
    """Creates symbolic link for a file into a new name file"""
    cmd = 'ln -s %s %s' %(file2link, newfile)
    system_call_functions.system_call(cmd, returned=False)


###############
def extract(fileGiven, out, remove=True):
    """
    Extracts archived file
    
    This function extracts the file given in the ``out`` path provided.
    It uses the ``patoolib`` that is able to identify the type of file 
    and compression algorithm to use in each case.
    
    It also removes the compressed file using ``os`` module.
    
    :param fileGiven: Archived file to extract.
    :param out: Output name and absolute path for the extracted archived.
    :param remove: True/False for removing compressed file extracted
    
    :type fileGiven: string
    :type out: string
    :type remove: boolean
    
    """
    ## extract using patoolib
    patoolib.extract_archive(fileGiven, outdir=out, verbosity=0) ## CHECK conda supported
    
    if (remove):
        ## remove compress file
        print ("Remove compress file...")
        os.remove(fileGiven)
        print ("\n")




#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
Shared functions used along ``BacterialTyper`` & ``XICRA`` pipeline.
With different purposes:
    - Print time stamps
  
    - Create system calls

    - Manage/list/arrange files/folders

    - Aesthetics

    - Manage fasta files

    - Other miscellaneous functions
"""
## useful imports
import time
from datetime import datetime

## my modules
from HCGB.functions import aesthetics_functions

###########################
##      TIME            ###
###########################

###############   
def print_time ():
    """Prints time stamp in human readable format: month/day/year, hour:minute:seconds."""
    now = datetime.now()
    date_time = now.strftime("%m/%d/%Y, %H:%M:%S")
    print ('\t' + date_time + '\n')

###############   
def gettime (start_time):
    """Obtains time stamp in human readable format: hour:minute:seconds from a time.time() format timestamp."""
    total_sec = time.time() - start_time
    m, s = divmod(int(total_sec), 60)
    h, m = divmod(m, 60)
    return h, m, s

###############    
def timestamp (start_time_partial):
    """Prints a stamp of the time spent for a process in human readable format: hour:minute:seconds.
    Returns time in format time.time().
    """
    h,m,s = gettime(start_time_partial)
    aesthetics_functions.print_sepLine("-", 25, False)
    print ('(Time spent: %i h %i min %i s)' %(int(h), int(m), int(s)))
    aesthetics_functions.print_sepLine("-", 25, False)
    return time.time()

###############    
def print_time_stamp (out):
    """Prints out timestamp in a file provided. Format: time.time()"""
    timefile = open(out, 'w')    
    string2write = str(time.time())
    timefile.write(string2write)
    return()

###############    
def read_time_stamp (out):
    """Reads timestamp from a file provided. Format: time.time()"""
    st_hd = open(out, 'r')
    st = st_hd.read()
    st_hd.close()
    stamp = datetime.fromtimestamp(float(st)).strftime('%Y-%m-%d %H:%M:%S')
    return(stamp)

###############    
def get_diff_time(stamp):
    """Obtains the time spent for a process in days given a stamp in time.time() format.
    Returns days passed since.
    """

    time_today = time.time()
    elapsed = time_today - float(time_today)
    days_passed = int((elapsed/3600)/24)
    return(days_passed)

###############    
def create_human_timestamp():
    """Generates human timestamp for the date of day in format (yearmonthday): e.g. 20191011"""
    now = datetime.now()
    timeprint = now.strftime("%Y%m%d")
    return timeprint
###############



#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
Shared functions used along ``BacterialTyper`` & ``XICRA`` pipeline.
With different purposes:
    - Print time stamps
  
    - Create system calls

    - Manage/list/arrange files/folders

    - Aesthetics

    - Manage fasta files

    - Other miscellaneous functions
"""
## useful imports
import re
from Bio import SeqIO
from termcolor import colored
from collections import defaultdict
##
from HCGB.functions import system_call_functions

#################################
###         FASTA files        ##
#################################

###############
def concat_fasta(dirFasta, Fasta):
    print ("+ Concatenating all information into one file...")    
    cmd = 'cat ' + dirFasta + '/*fna > ' + Fasta
    return(system_call_functions.system_call(cmd))

###############
def subset_fasta(ident, fasta, out):
    output_FASTA = open(out, 'w')    
    for record in SeqIO.parse(fasta, "fasta"):
        all_id = record.description
        species_search = re.search(r"%s" % ident, all_id)
        if species_search:
            head = ">" + all_id + "\n"
            output_FASTA.write(head)
            output_FASTA.write(str(record.seq))
            output_FASTA.write("\n")
    
    output_FASTA.close()

###############
def rename_fasta_seqs(fasta_file, name, new_fasta):
    """Rename fasta sequences provided in file :file:`fasta_file` using id :file:`name`. Save results in file :file:`new_fasta` provided.
    
    Check for id character lenght do not exceed 37 characters as it might be a limitiation in further annotation and subsequent analysis. Read Prokka_ issue for further details: https://github.com/tseemann/prokka/issues/337.
    
    :param fasta_file: Absolute path to fasta file.
    :type fasta_file: string
    :param name: String to add every fasta sequence header.
    :type name: string
    :param new_fasta: Name for the new fasta file (Absolute path).
    :type new_fasta: string
    :return: Path to tabular delimited file containing conversion from all to new id for each sequence.
    :warnings: Returns FAIL if name is >37 characters.
    
    .. include:: ../../links.inc         
    """

    output_FASTA = open(new_fasta, 'w')
    id_conversion = open(new_fasta + "_conversionID.txt", 'w')
    
    if len(name) > 37:
        print (colored("** ERROR **", 'red'))
        print (colored("rename_fasta_seqs():: name id is > 37 characters.", 'red'))
        print (colored("** ERROR **", 'red'))
        return ('FAIL')
    
    counter_seqs = 0
    for record in SeqIO.parse(fasta_file, "fasta"):
        old_id = record.description
        counter_seqs += 1
        new_id = name + "_" + str(counter_seqs)
        head = ">" + new_id + "\n"
        output_FASTA.write(head)
        output_FASTA.write(str(record.seq))
        output_FASTA.write("\n")
        
        id_conversion.write(old_id + "\t" + new_id)
        id_conversion.write("\n")
    
    output_FASTA.close()
    id_conversion.close()
        
    return (new_fasta + "_conversionID.txt")

### 
def process_fasta(lines):
    ks = ['name', 'sequence']
    return {k: v for k, v in zip(ks, lines)}

### 
def process_fastq(lines):
    ks = ['name', 'sequence', 'optional', 'quality']
    return {k: v for k, v in zip(ks, lines)}

#####
def reads2tabular(fastq_file, out):
    
    ## dictionary
    freq_fasta = defaultdict(int)
    
    ## read fastq    
    n = 4
    with open(fastq_file, 'r') as fh:
        lines = []
        for line in fh:
            lines.append(line.rstrip())
            if len(lines) == n:
                record = process_fastq(lines)
                #sys.stderr.write("Record: %s\n" % (str(record)))
                lines = []
                
                ## add sequences & count
                freq_fasta[record['sequence']] += 1

    ## print in file
    with open(out,'w') as file:
        for k in sorted (freq_fasta.keys()):
            file.write("%s\t%s\n" % (k, freq_fasta[k]))
    
    return(freq_fasta)







#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
Shared functions used along ``BacterialTyper`` & ``XICRA`` pipeline.
With different purposes:
    - Print time stamps
  
    - Create system calls

    - Manage/list/arrange files/folders

    - Aesthetics

    - Manage fasta files

    - Other miscellaneous functions
"""
## useful imports
import subprocess
import wget
from termcolor import colored
##from filehash import FileHash ## FIXME: not conda supported
import os

#################################
##         system call        ###
#################################

###############
def system_call(cmd, returned=False, message=True):
    """Generates system call using subprocess.check_output"""
    ## call system
    ## send command
    if (message):
        print (colored("[** System: %s **]" % cmd, 'magenta'))

    try:
        out = subprocess.check_output(cmd, shell = True)
        if (returned):
            return (out)
        return ('OK')
    except subprocess.CalledProcessError as err:
        if (returned):
            return (err.output)
        if (message):
            print (colored("** ERROR **", 'red'))
            print (colored(err.output, 'red'))
            print (colored("** ERROR **", 'red'))
        
        return ('FAIL')

###############
def wget_download(url, path):
    """
    Downloads file in the given path.
    
    It uses python module wget to download given ``url`` in the ``path`` provided.
    
    :param url: File to download from http/ftp site.
    :param path: Absolute path to the folder where to save the downloaded file.
    :type url: string
    :type path: string  
    
    :returns: Print messages and generates download
    """
    
    print ('\t+ Downloading: ', url)
    wget.download(url, path)
    print ('\n')

###############
def check_md5sum(string, File):
    ## FIXME: Not conda supported
    return(True)

#    md5hasher = FileHash('md5')
#    md5_file = md5hasher.hash_file(File) 
#    
#    if (md5_file == string):
#        #print (md5_file + '==' + string)
#        return (True)
#    else:
#        return (False)

###############
def chmod_rights(file, access_rights):
    """Changes access permission of a file:
    
    Read additional information in:
    https://docs.python.org/3/library/os.html#os.chmod
    https://www.geeksforgeeks.org/python-os-chmod-method/
    """
    access = os.chmod(file, access_rights, follow_symlinks=True)
    return(access)



#!/usr/bin/ python3
##########################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain        ##
##########################################################
"""
Prepares samples for further analysis.
"""

import os
import re
import pandas as pd
from termcolor import colored

from HCGB import sampleParser
from HCGB import functions

def select_samples (list_samples, samples_prefix, pair=True, exclude=False, Debug=False, lane=False, include_all=False):
    """
    Select samples
    
    Given a sample prefix (any or a given list), this function retrieves
    sample files from a list given. If exclude option provided, excludes 
    the files retrieved from the total.
    
    :param list_samples: List of absolute path for fastq files
    :param samples_prefix: List of prefix to search 
    :param pair: True/false for paired-end files
    :param exclude: True/false for exclude found files from the total
    :param Debug: True/false for debugging messages
    :param lane: Include lane tag within name id
    
    :type list_samples: list
    :type samples_prefix: list
    :type pair: bool
    :type exclude: bool
    :type Debug: bool
    :type lane: bool
    
    :returns: Dataframe
    """
    #Get all files in the folder "path_to_samples"
    sample_list = pd.DataFrame(columns=('sample', 'file'))
    
    for names in samples_prefix:
        for path_fastq in list_samples:    
            fastq = os.path.basename(path_fastq)
            samplename_search = re.search(r"(%s)\_{0,1}(R1|1|R2|2){0,1}(.*){0,1}\.f.*q.*" % names, fastq)
            enter = ""
            if samplename_search:
                if (exclude): ## exclude==True
                    enter = False
                else: ## exclude==True
                    enter = True
            else:
                if (exclude): ## exclude==True
                    enter = True
                else: ## exclude==True
                    enter = False
                    
            if enter:
                if fastq.endswith('.gz') or fastq.endswith('fastq') or fastq.endswith('fq'):
                    sample_list.loc[len(sample_list)] = (names, path_fastq) 
                else:
                    ## debug message
                    if (Debug):
                        print (colored("**DEBUG: sampleParser.select_samples **", 'yellow'))
                        print (colored("** ERROR: %s is a file that is neither in fastq.gz or .fastq format, so it is not included" %path_fastq, 'yellow'))
                            
    ## discard duplicates if any
    non_duplicate_names = sample_list['sample'].to_list() #
    non_duplicate_names = list(set(non_duplicate_names))
    
    ## it might be a bug in exclude list.
    ## if sample X1 is provided to be excluded, we might be also excluding
    ## sample X12, sample X13, etc.
    ## TODO: check this

    ## debugging messages
    if Debug:
        print (colored("** DEBUG: select_samples",'yellow'))
        print ("non_duplicate_names:")
        print (non_duplicate_names)
    
    ## check they match with given input
    if (exclude): ## exclude==True
        if bool(set(samples_prefix).intersection(non_duplicate_names)):
            print(colored("** ERROR: Some non desired samples are included", 'red'))
    else: ## exclude==True
        non_duplicate_names = set(samples_prefix).intersection(non_duplicate_names)

    ## get fields
    
    tmp = sample_list[ sample_list['sample'].isin(non_duplicate_names) ]
    non_duplicate_samples = tmp['file'].to_list()
    
    ## debugging messages
    if Debug:
        print (colored("** DEBUG: select_samples",'yellow'))
        print ("non_duplicate_names:")
        print (non_duplicate_names)
        print ("samples_prefix")
        print (samples_prefix)
        print ("non_duplicate_samples")
        print (non_duplicate_samples)
        print ("tmp dataframe")
        functions.main_functions.print_all_pandaDF(tmp)
        print(tmp)
                
    ## get info
    name_frame_samples = sampleParser.files.get_fields(non_duplicate_samples, pair, Debug, include_all)    
    number_files = name_frame_samples.index.size
    total_samples = set(name_frame_samples['name'].to_list())
    
    ##
    if (lane):
        ## include lane tag within name
        name_frame_samples['name'] = name_frame_samples['name'] + '_' + name_frame_samples['lane']
        name_frame_samples['new_name'] = name_frame_samples['name']
            
    ## debugging messages
    if Debug:
        print (colored("** DEBUG: select_samples",'yellow'))
        print ("name_frame_samples:")
        print (name_frame_samples)
        print ("number_files:")
        print (number_files)
        print ("total_samples:")
        print (total_samples)
    
    ### get some stats
    if (number_files == 0):
        print (colored("\n**ERROR: No samples were retrieved. Check the input provided\n",'red'))
        exit()
    print (colored("\t" + str(number_files) + " files selected...", 'yellow'))
    print (colored("\t" + str(len(total_samples)) + " samples selected...", 'yellow'))
    if (pair):
        print (colored("\tPaired-end mode selected...", 'yellow'))
    else:
        print (colored("\tSingle end mode selected...", 'yellow'))
    
    ## return info
    return (name_frame_samples)

###############
def select_other_samples (project, list_samples, samples_prefix, mode, extensions, exclude=False, Debug=False):

    ## init dataframe
    name_columns = ("sample", "dirname", "name", "ext", "tag")

    ## initiate dataframe
    df_samples = pd.DataFrame(columns=name_columns)

    ## debug message
    if (Debug):
        print (colored("**DEBUG: samples_prefix **", 'yellow'))
        print (samples_prefix)
        print (colored("**DEBUG: mode **", 'yellow'))
        print (mode)
        print (colored("**DEBUG: extensions **", 'yellow'))
        print (extensions)

    #Get all files in the folder "path_to_samples"    
    for names in samples_prefix:
        for path_file in list_samples:    
            f = os.path.basename(path_file)
            dirN = os.path.dirname(path_file)
            #samplename_search = re.search(r"(%s).*" % names, f)
            samplename_search = re.search(r"(%s).*" % names, path_file)
            
            enter = ""
            if samplename_search:
                if (exclude): ## exclude==True
                    enter = False
                else: ## exclude==True
                    enter = True
            else:
                if (exclude): ## exclude==True
                    enter = True
                else: ## exclude==True
                    enter = False
                    
            if enter:
                
                ## project mode:
                if project:
                    if mode == 'annot':
                        #### /path/to/folder/annot/name.faa
                        for ext in extensions:
                            f_search = re.search(r".*\/%s\/(.*)\.%s$" %(mode, ext), path_file)
                            if f_search:
                                file_name = f_search.group(1) 
                                df_samples.loc[len(df_samples)] = [path_file, dirN, file_name, ext, mode]    

                    elif mode== 'assembly':
                        #### name_assembly.faa
                        for ext in extensions:
                            f_search = re.search(r"(.*)\_%s\.%s$" %(mode, ext), f)
                            if f_search:
                                file_name = f_search.group(1) 
                                df_samples.loc[len(df_samples)] = [path_file, dirN, file_name, ext, mode]    

                    elif mode== 'mash':
                        #### name.sig
                        for ext in extensions:
                            f_search = re.search(r".*\/%s\/(.*)\.%s$" %(mode, ext), path_file)
                            if f_search:
                                file_name = f_search.group(1) 
                                df_samples.loc[len(df_samples)] = [path_file, dirN, file_name, ext, mode]    

                    else:
                        for ext in extensions:
                            f_search = re.search(r".*\/(.*)\/%s\/(.*)\_summary\.%s$" %(mode, ext), path_file)
                            if f_search:
                                ### get information
                                if mode == 'profile':
                                    name = f_search.group(1)
                                    db_name = f_search.group(2).split('_')[-1]
                                    if not name.startswith('report'):
                                        df_samples.loc[len(df_samples)] = [path_file, dirN, name, db_name, mode]    
    
                                elif mode == 'ident':
                                    name = f_search.group(1)
                                    df_samples.loc[len(df_samples)] = [path_file, dirN, name, 'csv', mode]
                                    
                            else:
                                #### common: path/sample_name/job/info.csv
                                for ext in extensions:
                                    f_search = re.search(r"(.*)\/%s\/(.*)\.%s$" %(mode, ext), path_file)
                                    if f_search:
                                        file_name = os.path.basename(f_search.group(1))
                                        df_samples.loc[len(df_samples)] = [path_file, dirN, file_name, ext, mode]
                                    else:
                                        f_search2 = re.search(r"(.*)\/%s\/%s$" %(mode, ext), path_file)
                                        if f_search2:
                                            file_name = os.path.basename(f_search2.group(1))
                                            df_samples.loc[len(df_samples)] = [path_file, dirN, file_name, ext, mode]    
                                            
                                    
                ## detached mode
                else:
                    for ext in extensions:
                        if f.endswith(ext):
                            file_name, ext1 = os.path.splitext(f)
                            df_samples.loc[len(df_samples)] = [path_file, dirN, file_name, ext1, mode]    

    ## debug message
    if (Debug):
        print (colored("**DEBUG: df_samples **", 'yellow'))
        print (df_samples)
        print ("test OK")
    
    ## print stats
    number_samples = df_samples.index.size
    if (number_samples == 0):
        print (colored("\n**ERROR: No samples were retrieved for this option. Continue processing...\n",'red'))
        return (df_samples)
    print (colored("\t" + str(number_samples) + " samples selected from the input provided...", 'yellow'))

    df_samples['new_name'] = df_samples['name']
    return (df_samples)



#!/usr/bin/env python3
############################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain   ##
############################################################
"""
Maths functions used along ``BacterialTyper`` & ``XICRA`` pipeline.
"""
## useful imports
import re

#####################
def percentage(percent, whole):
    if (percent == "0.00%"):
        return 0

    percent_search = re.search(r"(.*)%", percent)
    if percent_search:
        percent_int = float(percent_search.group(1))
    else:
        percent_int = percent
        
    value = (percent_int * whole) / 100.0
    return int(value)


import setuptools

long_description_text = ""
with open("README.md", "r") as fh:
    long_description_text = fh.read()

setuptools.setup(
    name="HCGB",
    version="0.3.7",

    author="Jose F. Sanchez-Herrero",
    description="Useful python functions",

    author_email="jfbioinformatics@gmail.com",

    long_description_content_type="text/markdown",
    long_description=long_description_text,

    url="https://github.com/HCGB-IGTP/HCGB_python_functions/",

    packages=setuptools.find_packages(),
    license='MIT License',

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pandas', 'termcolor', 'biopython', 'wget', 'xlsxwriter', 'patool'
    ],
)



#!/usr/bin/ python3
##########################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain        ##
##########################################################
"""
Get information from files
"""

import os
import re
import pandas as pd
from termcolor import colored

from HCGB import functions
from HCGB import sampleParser

###############
def get_fields(file_name_list, pair, Debug, include_all):
    """
    Get information from files
    
    Creates and returns a dataframe containing information for each sample for:
    "sample", "dirname", "name", "new_name", "name_len", "lane", "read_pair","lane_file","ext","gz", "tag"
    
    :param file_name_list: List of files to parse
    :param pair: True/false for Paired-end files
    :param Debug: True/false for debugging messages
    
    :type file_name_list: list
    :type pair: bool
    :type Debug: bool
    
    :returns: Pandas dataframe.
    
    """
    ## init dataframe
    name_columns = ("sample", "dirname", "name", "new_name", "name_len", 
                "lane", "read_pair","lane_file","ext","gz", "tag", "file")
    name_frame = pd.DataFrame(columns=name_columns)
    
    ## loop through list
    for path_files in file_name_list:
        ## get file name
        file_name = os.path.basename(path_files)
        dirN = os.path.dirname(path_files)
        trim_search = re.search(r".*trim.*", file_name)
        lane_search = re.search(r".*\_L\d+\_.*", file_name)
        ## get name
        if (include_all):
            if (pair):
                name_search = re.search(r"(.*)\_(R1|1|R2|2)\_{0,1}(.*)\.(f.*q)(\..*){0,1}", file_name)
            else:
                name_search = re.search(r"(.*)\.(f.*q)(\..*){0,1}", file_name)

        else:    
            if (pair):
                ## pair could be: R1|R2 or 1|2
                if (trim_search):
                    name_search = re.search(r"(.*)\_trim\_(R1|1|R2|2)\.(f.*q)(\..*){0,1}", file_name)
                    
                else:
                    ## Lane files: need to merge by file_name: 33i_S5_L004_R1_001.fastq.gz
                    ## lane should contain L00x            
                    if (lane_search):
                        name_search = re.search(r"(.*)\_(L\d+)\_(R1|1|R2|2)\_{0,1}(.*)\.(f.*q)(\..*){0,1}", file_name)
                    else:
                        name_search = re.search(r"(.*)\_(R1|1|R2|2)\.(f.*q)(\..*){0,1}", file_name)
            else:
                if (trim_search):
                    name_search = re.search(r"(.*)\_trim(.*)\.(f.*q)(\..*){0,1}", file_name) ## trim.fastq or trim_joined.fastq
                else:
                    name_search = re.search(r"(.*)\.(f.*q)(\..*){0,1}", file_name)
        
        ### declare
        name= ""
        lane_id= ""
        read_pair= ""
        lane_file= ""
        ext= ""
        gz= ""
    
        if name_search:
            name = name_search.group(1)
            name_len = len(name)
            if (pair):
                if (include_all):
                    lane_id = ""
                    read_pair = name_search.group(2)
                    lane_file = name_search.group(3)
                    ext = name_search.group(4)
                    gz = name_search.group(5)
                                
                ## Lane files: need to merge by file_name: 33i_S5_L004_R1_001.fastq.gz
                elif (lane_search):
                    lane_id = name_search.group(2)
                    read_pair = name_search.group(3)
                    lane_file = name_search.group(4)
                    ext = name_search.group(5)
                    gz = name_search.group(6)
                else:
                    ## could exist or not
                    read_pair = name_search.group(2)
                    ext = name_search.group(3)
                    gz = name_search.group(4)
            else:
                ext = name_search.group(2)
                gz = name_search.group(3)
    
            ## populate dataframe
            name_frame.loc [len(name_frame)] = (path_files, dirN, name, name, name_len, 
                                            lane_id, read_pair, lane_file, ext, gz, "reads", os.path.basename(path_files))
    
        else:
            ## debug message
            if (Debug):
                print (colored("**DEBUG: sampleParser.get_fields **", 'yellow'))
                print (colored("*** ATTENTION: Sample did not match the possible parsing options...", 'yellow'))
                print (file_name)
            
            print (colored("*** ATTENTION: Sample (%s) did not match the possible parsing options..." %path_files, 'yellow'))

    return (name_frame)

###############

################################
def get_files(options, input_dir, mode, extension, debug):
    """
    Parser function to get files
    
    Given an input dir and a mode in retrieves
    matching files with the extension desired.
    
    :param options: Contains several options as parser.parse_args options.
    :param input_dir: Absolute path to input dir containing samples.
    :param mode: Options are: fastq, trim, annot, assembly, etc...
    :param extension: List of possible extension to retrieve.
    :param debug
    
    :type options: parser.parse_args
    :type input_dir: string 
    :type mode: string 
    :type extension: list
    
    :returns: Pandas dataframe with sample and file information.
    
    options contains:
        + options.debug bool
        + options.project bool
        + options.batch bool
        + options.pair bool
        + options.include_all bool
        + options.include_lane bool
        + options.in_sample file or string
        + options.ex_sample file or string    
    
    """
    ## get list of input files
    files = []
    if (options.project):
        
        ## input folder is not a dir, is it a batch input file?
        if (options.batch):
            if os.path.isfile(input_dir):
                if (options.debug):
                    print (colored("\n**DEBUG: sampleParser.get_files input folder is a batch file, get full path **", 'yellow'))
                dir_list = [line.rstrip('\n') for line in open(input_dir)]
                for d in dir_list:
                    if os.path.exists(d):
                        print ('+ Folder (%s) exists' %d)
                        files = files + functions.main_functions.get_fullpath_list(d, debug)
                    else:
                        ## input folder does not exist...
                        if (options.debug):
                            print (colored("\n**DEBUG: sampleParser.get_files batch option; input folder does not exists **", 'yellow'))
                            print (d)
                            print ("\n")
        else:        
            ### a folder containing a project is provided
            if os.path.exists(input_dir):
                print ('+ Input folder exists')
                ## get files in folder
                for ext in extension:
                    if mode == 'trim':
                        files_tmp = functions.main_functions.get_fullpath_list(input_dir, debug)
                        files = [s for s in files_tmp if ext in s]
                    else:
                        files_tmp = functions.main_functions.retrieve_matching_files(input_dir, ext, debug)
                        files = files + files_tmp
    
                files = set(files)

            else:
                ## input folder does not exist...
                if (options.debug):
                    print (colored("\n**DEBUG: sampleParser.get_files input folder does not exists **", 'yellow'))
                    print (input_dir)
                    print ("\n")
    
                print (colored('***ERROR: input folder does not exist or it is not readable', 'red'))
                exit()                        
    else:
        ### provide a single folder or a file with multiple paths (option batch)
        if (options.batch):
            if os.path.isfile(input_dir):
                dir_list = [line.rstrip('\n') for line in open(input_dir)]
                for d in dir_list:
                    if os.path.exists(d):
                        print ('+ Folder (%s) exists' %d)
                        files_tmp = functions.main_functions.get_fullpath_list(d, debug)
                        files = files + files_tmp
            
                    else:
                        ## input folder does not exist...
                        if (options.debug):
                            print (colored("\n**DEBUG: sampleParser.get_files batch option; input folder does not exists **", 'yellow'))
                            print (d)
                            print ("\n")
                        
        else:
            if os.path.exists(input_dir):
                print ('+ Input folder exists')
                ## get files in folder
                files = functions.main_functions.get_fullpath_list(input_dir, debug)
            else:
                ## input folder does not exist...
                if (options.debug):
                    print (colored("\n**DEBUG: sampleParser.get_files input folder does not exists **", 'yellow'))
                    print (input_dir)
                    print ("\n")
    
                print (colored('***ERROR: input folder does not exist or it is not readable', 'red'))
                exit()

    if options.debug:
        print (colored("** DEBUG: sampleParser.get_files files", 'yellow'))
        print (files)

    ## get list of samples
    samples_names = []
    exclude=False

    if (options.in_sample):
        if os.path.isfile(os.path.abspath(options.in_sample)): ## a file
            in_file = os.path.abspath(options.in_sample)
            samples_names = [line.rstrip('\n') for line in open(in_file)]
            print ('+ Retrieve selected samples to obtain from the list files available.')
        else: ## a single ID
            samples_names = [options.in_sample]
        
        exclude=False

        ## in sample list...
        if (options.debug):
            print (colored("\n**DEBUG: sampleParser.get_files include sample list **", 'yellow'))
            print (samples_names, '\n')


    elif (options.ex_sample):
        if os.path.isfile(os.path.abspath(options.ex_sample)): ## a file
            ex_file = os.path.abspath(options.ex_sample)
            samples_names = [line.rstrip('\n') for line in open(ex_file)]
            print ('+ Retrieve selected samples to exclude from the list files available.')        
            exclude=True
        else: ## a single ID
            samples_names = [options.ex_sample]
        
        ## in sample list...
        if (options.debug):
            print (colored("\n**DEBUG: sampleParser.get_files exclude sample list **", 'yellow'))
            print (samples_names, '\n')

    else:
        samples_names = ['.*']

    ## discard empty sample_names
    samples_names = list(filter(None, samples_names)) ## empty space

    ## discard some files obtain
    files = [s for s in files if '.bam' not in s]
    files = [s for s in files if '.sam' not in s]
    files = [s for s in files if '.log' not in s]
    files = [s for s in files if '.annot' not in s]
    files = [s for s in files if '.abundances.txt' not in s]
    files = [s for s in files if '.gff3' not in s]
    files = [s for s in files if 'trimmed.fq' not in s]
    files = [s for s in files if 'trim.clpsd.fq' not in s]
    files = [s for s in files if 'failed.fq.gz' not in s]
    files = [s for s in files if 'unjoin' not in s]
    files = [s for s in files if '00.0_0.cor.fastq.gz' not in s]
    
    if (mode == 'fastq'):
        files = [s for s in files if 'trim' not in s]
    
    ##
    files = list(filter(None, files)) ## empty space
        
    ## files list...
    if (options.debug):
        print (colored("\n**DEBUG: sampleParser.get_files files list to check **", 'yellow'))
        print ('DO NOT PRINT THIS LIST: It could be very large...')
        ##print (files, '\n')

    ## get information
    if mode in ['fastq', 'trim', 'join']:
        pd_samples_retrieved = sampleParser.samples.select_samples(files, samples_names, options.pair, exclude, options.debug, options.include_lane, options.include_all)
    else:
        pd_samples_retrieved = sampleParser.samples.select_other_samples(options.project, files, samples_names, mode, extension, exclude, options.debug)        
    
    ## remove duplicates & return
    pd_samples_retrieved = pd_samples_retrieved.drop_duplicates()
    
    return(pd_samples_retrieved)
